`include "ComputeUnit.v"

module top;

  reg reset;

  reg clk = 1'b1;
  always begin
    #5;
    clk = ~clk;
  end

  reg [31:0] cycle_num = 32'b0;
  always @ ( posedge clk ) begin
    cycle_num = cycle_num + 1'b1;
  end

  // Memory

  wire [16:0] read_mem_addr;
  wire [7:0]  read_mem_data;

  wire [7:0]  y = read_mem_addr[16:9];
  wire [8:0]  x = read_mem_addr[8:0];

  FakeMemoryForRead read_memory
  (
    .clk(clk),
    .reset(reset),

    .addr(read_mem_addr),
    .data(read_mem_data)
  );

  // DUT instantiation

  reg writing_useful_location = 1'b1;
  reg compute_go = 1'b0;

  wire [16:0] compute_and_decoder_m9k_addr;
  wire [7:0]  compute_and_decoder_m9k_data;
  wire        compute_and_decoder_m9k_wren;

  wire [9:0]  compute_and_bird_seek_m9k_data;
  wire        compute_and_bird_seek_m9k_wren;

  wire [9:0]  compute_and_horz_seek_m9k_data;
  wire        compute_and_horz_seek_m9k_wren;

  wire [9:0]  compute_and_vert_seek_m9k_data;
  wire        compute_and_vert_seek_m9k_wren;

  wire [2:0]  tap_times;
  wire        tap_times_val;

  ComputeUnit compute_unit
  (
    .clk                            (clk),
    .reset                          (reset),

    .compute_go                     (compute_go),

    // Configuration
    .bird_threshold                 (16'd15),
    .bird_y                         (8'd200),

    .pipe_0_threshold               (16'd7),
    .pipe_scan_x                    (9'd50),

    .bird_lower_limit               (9'd50),

    // Camera decoder m9k side
    .decoder_m9k_and_compute_addr   (read_mem_addr),
    .decoder_m9k_and_compute_data   (read_mem_data),

    // VGA m9k side
    .compute_and_decoder_m9k_addr   (compute_and_decoder_m9k_addr),
    .compute_and_decoder_m9k_data   (compute_and_decoder_m9k_data),
    .compute_and_decoder_m9k_wren   (compute_and_decoder_m9k_wren),

    // Output to seek m9k blocks
    .compute_and_bird_seek_m9k_data (compute_and_bird_seek_m9k_data),
    .compute_and_bird_seek_m9k_wren (compute_and_bird_seek_m9k_wren),

    .compute_and_horz_seek_m9k_data (compute_and_horz_seek_m9k_data),
    .compute_and_horz_seek_m9k_wren (compute_and_horz_seek_m9k_wren),

    .compute_and_vert_seek_m9k_data (compute_and_vert_seek_m9k_data),
    .compute_and_vert_seek_m9k_wren (compute_and_vert_seek_m9k_wren),

    .tap_times                      (tap_times),
    .tap_times_val                  (tap_times_val)
  );

  initial begin
    $dumpfile("ComputeUnit.vcd");
    $dumpvars(0, top);

    #1; reset = 1'b1;
    #10; reset = 1'b0;

    #200;

    compute_go = 1'b1;
    #10
    compute_go = 1'b0;

    #15000;

    $finish;
  end

endmodule


module FakeMemoryForRead
(
 input            clk,
 input            reset,

 input [16:0] addr,
 output reg [7:0] data
);

  wire [8:0]      x = addr[8:0];
  wire [9:0]      y = addr[16:9];

  always @ ( posedge clk ) begin
    if ( y == 200 ) begin
      // For the bird
      if ( x > 100 )
        data <= 8'h33;
      else
        data <= 8'haa;
    end else if ( x == 50 ) begin
      // For HorzSeek
      if ( y < 80 )
        data <= 8'h33;
      else
        data <= 8'haa;
    end else if ( y == 53 ) begin
      if ( x > 60 )
        data <= 8'h33;
      else
        data <= 8'haa;
    end else begin
      data <= x[7:0];
    end
  end

endmodule
