`ifndef COPIER_V
`define COPIER_V

`include "vc-Counter.v"

module Copier
(
 input         clk,
 input         reset,
 input         pause,

 output [16:0] decoder_side_m9k_addr,
 input [7:0]   decoder_side_m9k_data,

 output [16:0] vga_side_m9k_addr,
 output [7:0]  vga_side_m9k_data,
 output        vga_side_m9k_wren
);

  reg          x_increment;
  reg          x_clear;
  wire [8:0]   x;
  wire         x_is_max;

  vc_Counter
  #(
    .p_count_nbits       (9),
    .p_count_reset_value (9'b0),
    .p_count_max_value   (9'd511)
  )
  x_counter
  (
   .clk           (clk),
   .reset         (reset || x_clear),

   .increment     (x_increment),
   .decrement     (1'b0),

   .count         (x),
   .count_is_max  (x_is_max),
   .count_is_zero ()
  );

  reg          y_increment;
  reg          y_clear;
  wire [7:0]   y;
  wire         y_is_max;


  vc_Counter
  #(
    .p_count_nbits       (8),
    .p_count_reset_value (8'b0),
    .p_count_max_value   (8'd255)
  )
  y_counter
  (
   .clk           (clk),
   .reset         (reset || y_clear),

   .increment     (y_increment),
   .decrement     (1'b0),

   .count         (y),
   .count_is_max  (y_is_max),
   .count_is_zero ()
  );

  assign decoder_side_m9k_addr = { y, x };
  assign vga_side_m9k_addr = { y, x };
  assign vga_side_m9k_data = decoder_side_m9k_data;
  assign vga_side_m9k_wren = ~pause;

  // Control logic
  always @ ( * ) begin
    if ( pause ) begin
      x_increment = 1'b0;
      x_clear = 1'b0;
      y_increment = 1'b0;
      y_clear = 1'b0;
    end else begin
      if ( !x_is_max ) begin
        x_increment = 1'b1;
        x_clear = 1'b0;
        y_increment = 1'b0;
        y_clear = 1'b0;
      end else begin
        // x_is_max
        if ( ~y_is_max ) begin
          x_increment = 1'b0;
          x_clear = 1'b1;
          y_increment = 1'b1;
          y_clear = 1'b0;
        end else begin
          x_increment = 1'b0;
          x_clear = 1'b1;
          y_increment = 1'b0;
          y_clear = 1'b1;
        end
      end
    end
  end

endmodule

`endif
