module  ITU_656_Decoder
(
 //  TV Decoder Input
 input [7:0]   iTD_DATA,
 //  Position Output
 output [9:0]  oTV_X,
 output [9:0]  oTV_Y,
 output [31:0] oTV_count,
 //  YUV 4:2:2 Output
 output [15:0] oYCbCr,
 output        oDVAL,
 //  Control Signals
 input         iSwap_CbCr,
 input         iSkip,
 input         iRST_N,
 input         iCLK_27,

 // High when writing frame
 output reg    FVAL,
 output reg    Field
);

  //  For detection
  reg [23:0]   Window;   //  Sliding window register
  reg [17:0]   count;     //  Counter
  reg          Active_Video;
  reg          Start;
  reg          Data_Valid;
  reg          Pre_Field;
  wire         SAV;
  reg [9:0]    y_field_0;
  reg [9:0]    y_field_1;
  reg [31:0]   Data_count;

  //  For ITU-R 656 to ITU-R 601
  reg [7:0]    Cb;
  reg [7:0]    Cr;
  reg [15:0]   YCbCr;

  wire [17:0] count_half = count >> 1;

  assign oTV_X = count_half[9:0];
  assign oTV_Y = ( ~Field ) ? y_field_0 : y_field_1;
  assign oYCbCr  = YCbCr;
  assign oDVAL = Data_Valid;
  assign oTV_count = Data_count;

  assign SAV   = (Window==24'hFF0000)&(iTD_DATA[4]==1'b0);

  always @ (posedge iCLK_27 or negedge iRST_N) begin
    if(!iRST_N) begin
      //  Register initial
      Active_Video <= 1'b0;
      Start        <= 1'b0;
      Data_Valid   <= 1'b0;
      Pre_Field    <= 1'b0;
      Field        <= 1'b0;
      Window       <= 24'h0;
      count        <= 18'h0;
      Cb           <= 8'h0;
      Cr           <= 8'h0;
      YCbCr        <= 16'h0;
      FVAL         <= 1'b0;
      y_field_0    <= 10'b0;
      y_field_1    <= 10'b1;
      Data_count   <= 32'h0;
    end else begin
      //  Sliding window
      Window  <=  {Window[15:0],iTD_DATA};

      //  Active data counter
      if(SAV)
        count  <=  18'h0;
      else if(count < 18'd1440)
        count  <=  count + 1'b1;

      //  Check the video data is active?
      if(SAV)
        Active_Video <= 1'b1;
      else if(count==1440)
        Active_Video <= 1'b0;

      //  Is frame start?
      Pre_Field <=  Field;
      if({Pre_Field,Field}==2'b10)
        Start   <=  1'b1;

      //  Field and frame valid check
      if(Window==24'hFF0000) begin
        FVAL  <=  !iTD_DATA[5];
        Field <=  iTD_DATA[6];
      end

      //  ITU-R 656 to ITU-R 601
      if(iSwap_CbCr) begin
        case(count[1:0])   //  Swap
          0:  Cb    <=   iTD_DATA;
          1:  YCbCr <=  {iTD_DATA,Cr};
          2:  Cr    <=   iTD_DATA;
          3:  YCbCr <=  {iTD_DATA,Cb};
        endcase
      end else begin
        case(count[1:0])   //  Normal
          0:  Cb    <=   iTD_DATA;
          1:  YCbCr <=  {iTD_DATA,Cb};
          2:  Cr    <=   iTD_DATA;
          3:  YCbCr <=  {iTD_DATA,Cr};
        endcase
      end

      //  Check data valid
      if( Start            //  Frame Start?
          &&  FVAL         //  Frame valid?
          &&  Active_Video //  Active video?
          &&  count[0]      //  Complete ITU-R 601?
          &&  !iSkip  )    //  Is non-skip pixel?
        Data_Valid  <=  1'b1;
      else
        Data_Valid  <=  1'b0;

      //  TV decoder line counter for one field
      if(FVAL && SAV)
        if ( ~Field )
          y_field_0 <= y_field_0 + 2'd2;
        else
          y_field_1 <= y_field_1 + 2'd2;

      if(!FVAL) begin
        y_field_0 <= 10'b0;
        y_field_1 <= 10'b1;
      end

      //  Data counter for one field
      if(!FVAL)
        Data_count <= 32'b0;
      if(Data_Valid)
        Data_count <= Data_count + 1'b1;
    end
  end

endmodule
