`ifndef TAPORNOT_V
`define TAPORNOT_V

`include "vc-regs.v"
`include "MathComponent.v"

module TapOrNot
(
 // Configuration
 input [8:0]      bird_lower_limit,

 // Inputs from scanning
 input            bird_top_x_0_valid,
 input [8:0]      bird_top_x_0,

 input            pipe_0_y_0_valid,
 input [7:0]      pipe_0_y_0,

 input            pipe_0_gap_top_x_0_valid,
 input [8:0]      pipe_0_gap_top_x_0,

 // Inputs from scanning
 input            bird_top_x_1_valid,
 input [8:0]      bird_top_x_1,

 input            pipe_0_y_1_valid,
 input [7:0]      pipe_0_y_1,

 input            pipe_0_gap_top_x_1_valid,
 input [8:0]      pipe_0_gap_top_x_1,

 output [8:0]     bird_threshold_display,

 output [2:0]     tap_times,
 output [8:0]     predicted_position,

 output [8:0]     debug
);

  //--------------------------------------------------------------------
  // MathComponent - does the math for this module
  //--------------------------------------------------------------------

  wire            valid_change;

  MathComponent math_component
  (
    .bird_top_x_0       (bird_top_x_0),
    .bird_top_x_1       (bird_top_x_1),
    .predicted_position (predicted_position),
    .valid_change       (valid_change)
  );

  //--------------------------------------------------------------------
  // Decision making
  //--------------------------------------------------------------------
  wire [8:0]      center_between_pipes = pipe_0_gap_top_x_1 - 9'd60;
  wire [8:0]      eventual_threshold = (bird_top_x_1_valid && pipe_0_y_1_valid && pipe_0_gap_top_x_1_valid) ? center_between_pipes : bird_lower_limit;  

  wire            all_valid = ( bird_top_x_0_valid
                                && pipe_0_y_0_valid
                                && pipe_0_gap_top_x_0_valid
                                && bird_top_x_1_valid
                                && pipe_0_y_1_valid
                                && pipe_0_gap_top_x_1_valid );

  wire [8:0]      intended_threshold = eventual_threshold;

  wire signed [10:0] signed_predicted_position = predicted_position;
  wire signed [10:0] signed_intended_threshold = intended_threshold;
  wire signed [10:0] signed_difference = signed_predicted_position - signed_intended_threshold;

  wire               positive_difference = ~signed_difference[10];
  wire signed [10:0] abs_difference = positive_difference ? signed_difference : -signed_difference;

  assign tap_times = /*valid_change && */positive_difference ?
                     abs_difference[9:7] + 3'd1 : 3'b0;

  assign bird_threshold_display = intended_threshold;

endmodule

`endif
