// --------------------------------------------------------------------
// ECE 5760 Lab5
// --------------------------------------------------------------------

module camera (
  //= Clock Input
  input         CLOCK_50,       // 50 MHz
  input         CLOCK2_50,      // 50 MHz
  input         CLOCK3_50,      // 50 MHz
  //= Push Button
  input [3:0]   KEY,            // Pushbutton[3:0]
  //= DPDT Switch
  input [17:0]  SW,             // Toggle Switch[17:0]
  //= 7-SEG Display
  output [6:0]  HEX0,           // Seven Segment Digit 0
  output [6:0]  HEX1,           // Seven Segment Digit 1
  output [6:0]  HEX2,           // Seven Segment Digit 2
  output [6:0]  HEX3,           // Seven Segment Digit 3
  output [6:0]  HEX4,           // Seven Segment Digit 4
  output [6:0]  HEX5,           // Seven Segment Digit 5
  output [6:0]  HEX6,           // Seven Segment Digit 6
  output [6:0]  HEX7,           // Seven Segment Digit 7
  //= LED
  output [8:0]  LEDG,           // LED Green[8:0]
  output [17:0] LEDR,           // LED Red[17:0]
  //= UART
  output        UART_CTS,       // UART Clear To Send
  input         UART_RTS,       // UART Request To Send
  output        UART_TXD,       // UART Transmitter
  input         UART_RXD,       // UART Receiver
  //= IRDA
  input         IRDA_RXD,       // IRDA Receiver
  //= SDRAM Interface
  inout [31:0]  DRAM_DQ,        // SDRAM Data bus 32 Bits
  output [12:0] DRAM_ADDR,      // SDRAM Address bus 13 Bits
  output        DRAM_WE_N,      // SDRAM Write Enable
  output        DRAM_CAS_N,     // SDRAM Column Address Strobe
  output        DRAM_RAS_N,     // SDRAM Row Address Strobe
  output        DRAM_CS_N,      // SDRAM Chip Select
  output        DRAM_CLK,       // SDRAM Clock
  output        DRAM_CKE,       // SDRAM Clock Enable
  output [1:0]  DRAM_BA,        // SDRAM Bank Address 2 Bits
  output [3:0]  DRAM_DQM,       // SDRAM Byte Data Mask 4 Bits
  //= Flash Interface
  inout [7:0]   FL_DQ,          // FLASH Data bus 8 Bits
  output [22:0] FL_ADDR,        // FLASH Address bus 23 Bits
  output        FL_WE_N,        // FLASH Write Enable
  output        FL_OE_N,        // FLASH Output Enable
  output        FL_CE_N,        // FLASH Chip Enable
  output        FL_RST_N,       // FLASH Reset
  input         FL_RY,          // FLASH Ready/Busy output
  output        FL_WP_N,        // FLASH Write Protect /Programming Acceleration
  //= SRAM Interface
  inout [15:0]  SRAM_DQ,        // SRAM Data bus 16 Bits
  output [19:0] SRAM_ADDR,      // SRAM Address bus 20 Bits
  output        SRAM_UB_N,      // SRAM High-byte Data Mask
  output        SRAM_LB_N,      // SRAM Low-byte Data Mask
  output        SRAM_WE_N,      // SRAM Write Enable
  output        SRAM_CE_N,      // SRAM Chip Enable
  output        SRAM_OE_N,      // SRAM Output Enable
  //= ISP1362 Interface
  inout [15:0]  OTG_DATA,       // ISP1362 Data bus 16 Bits
  output [1:0]  OTG_ADDR,       // ISP1362 Address 2 Bits
  output        OTG_CS_N,       // ISP1362 Chip Select
  output        OTG_WR_N,       // ISP1362 Write
  output        OTG_RD_N,       // ISP1362 Read
  input [1:0]   OTG_INT,        // ISP1362 Interrupt
  output        OTG_RST_N,      // ISP1362 Reset
  input [1:0]   OTG_DREQ,       // ISP1362 DMA Request
  output [1:0]  OTG_DACK_N,     // SP1362 DMA Acknowledge
  inout         OTG_FSPEED,     // USB Full Speed, 0 = Enable, Z = Disable
  inout         OTG_LSPEED,     // USB Low Speed,  0 = Enable, Z = Disable
  //= LCD Module 16X2
  inout [7:0]   LCD_DATA,       // LCD Data bus 8 bits
  output        LCD_ON,         // LCD Power ON/OFF
  output        LCD_BLON,       // LCD Back Light ON/OFF
  output        LCD_RW,         // LCD Read/Write Select, 0 = Write, 1 = Read
  output        LCD_EN,         // LCD Enable
  output        LCD_RS,         // LCD Command/Data Select, 0 = Command, 1 = Data
  //= SD Card Interface
  inout [3:0]   SD_DAT,         // SD Card Data 4 Bits
  inout         SD_CMD,         // SD Card Command Signal
  output        SD_CLK,         // SD Card Clock
  input         SD_WP_N,        // SD Write Protect
  //= EEPROM
  output        EEP_I2C_SCLK,   // EEPROM Clock
  inout         EEP_I2C_SDAT,   // EEPROM Data
  //= I2C
  inout         I2C_SDAT,       // I2C Data
  output        I2C_SCLK,       // I2C Clock
  //= VGA
  output        VGA_BLANK_N,    // VGA BLANK
  output        VGA_SYNC_N,     // VGA SYNC
  output        VGA_CLK,        // VGA Clock
  output        VGA_HS,         // VGA H_SYNC
  output        VGA_VS,         // VGA V_SYNC
  output [7:0]  VGA_R,          // VGA Red 8 Bits
  output [7:0]  VGA_G,          // VGA Green 8 Bits
  output [7:0]  VGA_B,          // VGA Blue 8 Bits
  //= Ethernet Clock
  input         ENETCLK_25,     // Ethernet Clock Source
  //= Ethernet Interface 0
  output        ENET0_GTX_CLK,  // GMII Transmit Clock
  input         ENET0_INT_N,    // Interrupt open drain output
  output        ENET0_MDC,      // Management data clock reference
  inout         ENET0_MDIO,     // Management data
  output        ENET0_RST_N,    // Hardware reset signal
  input         ENET0_RX_CLK,   // GMII and MII receive clock
  input         ENET0_RX_COL,   // GMII and MII collision
  input         ENET0_RX_CRS,   // GMII and MII carrier sense
  input [3:0]   ENET0_RX_DATA,  // GMII and MII receive data
  input         ENET0_RX_DV,    // GMII and MII receive data valid
  input         ENET0_RX_ER,    // GMII and MII receive error
  input         ENET0_TX_CLK,   // MII transmit clock
  output [3:0]  ENET0_TX_DATA,  // MII transmit data
  output        ENET0_TX_EN,    // GMII and MII transmit enable
  output        ENET0_TX_ER,    // GMII and MII transmit error
  input         ENET0_LINK100,  // Parallel LED output of 100BASE-TX link
  //= Ethernet Interface 1
  output        ENET1_GTX_CLK,  // GMII Transmit Clock
  input         ENET1_INT_N,    // Interrupt open drain output
  output        ENET1_MDC,      // Management data clock reference
  inout         ENET1_MDIO,     // Management data
  output        ENET1_RST_N,    // Hardware reset signal
  input         ENET1_RX_CLK,   // GMII and MII receive clock
  input         ENET1_RX_COL,   // GMII and MII collision
  input         ENET1_RX_CRS,   // GMII and MII carrier sense
  input [3:0]   ENET1_RX_DATA,  // GMII and MII receive data
  input         ENET1_RX_DV,    // GMII and MII receive data valid
  input         ENET1_RX_ER,    // GMII and MII receive error
  input         ENET1_TX_CLK,   // MII transmit clock
  output [3:0]  ENET1_TX_DATA,  // MII transmit data
  output        ENET1_TX_EN,    // GMII and MII transmit enable
  output        ENET1_TX_ER,    // GMII and MII transmit error
  input         ENET1_LINK100,  // Parallel LED output of 100BASE-TX link
  //= HSMC (LVDS)
//  input         HSMC_CLKIN0,    // Dedicated Clock Input
//  output        HSMC_CLKOUT0,   // Dedicated Clock Output
//  inout [3:0]   HSMC_D,         // Function depends on JP7
//  input         HSMC_CLKIN_P1,  // Function depends on JP7
//  input         HSMC_CLKIN_P2,  // Function depends on JP7
//  output        HSMC_CLKOUT_P1, // Function depends on JP7
//  output        HSMC_CLKOUT_P2, // Function depends on JP7
//  input [16:0]  HSMC_RX_D_P,    // Function depends on JP7
//  output [16:0] HSMC_TX_D_P,    // Function depends on JP7
//  input         HSMC_CLKIN_N1,  // Function depends on JP7
//  input         HSMC_CLKIN_N2,  // Function depends on JP7
//  output        HSMC_CLKOUT_N1, // Function depends on JP7
//  output        HSMC_CLKOUT_N2, // Function depends on JP7
//  input [16:0]  HSMC_RX_D_N,    // Function depends on JP7
//  output [16:0] HSMC_TX_D_N,    // Function depends on JP7
  //= Audio CODEC
  inout         AUD_ADCLRCK,    // Audio CODEC ADC LR Clock
  input         AUD_ADCDAT,     // Audio CODEC ADC Data
  inout         AUD_DACLRCK,    // Audio CODEC DAC LR Clock
  output        AUD_DACDAT,     // Audio CODEC DAC Data
  inout         AUD_BCLK,       // Audio CODEC Bit-Stream Clock
  output        AUD_XCK,        // Audio CODEC Chip Clock
  //= TV Decoder
  input         TD_CLK27,       // TV Decoder Clock Input
  output        TD_RESET_N,     // TV Decoder Reset
  input [7:0]   TD_DATA,        // TV Decoder Data bus 8 bits
  input         TD_HS,          // TV Decoder H_SYNC
  input         TD_VS,          // TV Decoder V_SYNC
  //= PS2 Mouse and Keyboard
  inout         PS2_CLK,        // PS/2 Clock
  inout         PS2_DAT,        // PS/2 Data
  inout         PS2_CLK2,       // PS/2 Clock
  inout         PS2_DAT2,       // PS/2 Data
  //= SMA
  input         SMA_CLKIN,      // External (SMA) clock input
  output        SMA_CLKOUT,     // External (SMA) clock output
  //= GPIO
  inout [35:0]  GPIO
);

  // Turn off 7-SEG Display
  assign HEX0           = 7'b0000000;
  assign HEX1           = 7'b0000000;
  assign HEX2           = 7'b0000000;
  assign HEX3           = 7'b0000000;
  assign HEX4           = 7'b0000000;
  assign HEX5           = 7'b0000000;
  assign HEX6           = 7'b0000000;
  assign HEX7           = 7'b0000000;

  // Turn off LED
  // assign LEDG           = 9'b000000000;
  // assign LEDR           = 18'b000000000000000000;

  // Turn off UART
  assign UART_CTS       = 1'b0;
  assign UART_TXD       = 1'b0;

  // Turn off SDRAM Interface
  assign DRAM_DQ        = 32'hzzzzzzzz;
  assign DRAM_ADDR      = 13'b0000000000000;
  assign DRAM_WE_N      = 1'b0;
  assign DRAM_CAS_N     = 1'b0;
  assign DRAM_RAS_N     = 1'b0;
  assign DRAM_CS_N      = 1'b0;
  assign DRAM_CLK       = 1'b0;
  assign DRAM_CKE       = 1'b0;
  assign DRAM_BA        = 2'b00;
  assign DRAM_DQM       = 4'b0000;

  // Turn off Flash Interface
  assign FL_DQ          = 8'hzz;
  assign FL_ADDR        = 23'b00000000000000000000000;
  assign FL_WE_N        = 1'b0;
  assign FL_OE_N        = 1'b0;
  assign FL_CE_N        = 1'b0;
  assign FL_RST_N       = 1'b0;
  assign FL_WP_N        = 1'b0;

  // Turn off SRAM Interface
  assign SRAM_DQ        = 16'hzzzz;
  assign SRAM_ADDR      = 20'b00000000000000000000;
  assign SRAM_UB_N      = 1'b1;
  assign SRAM_LB_N      = 1'b1;
  assign SRAM_WE_N      = 1'b1;
  assign SRAM_CE_N      = 1'b1;
  assign SRAM_OE_N      = 1'b1;

  // Turn off ISP1362 Interface
  assign OTG_DATA       = 16'hzzzz;
  assign OTG_ADDR       = 2'b00;
  assign OTG_CS_N       = 1'b0;
  assign OTG_WR_N       = 1'b0;
  assign OTG_RD_N       = 1'b0;
  assign OTG_RST_N      = 1'b0;
  assign OTG_DACK_N     = 2'b00;
  assign OTG_FSPEED     = 1'hz;
  assign OTG_LSPEED     = 1'hz;

  // Turn off LCD Module 16X2
  assign LCD_DATA       = 8'hzz;
  assign LCD_ON         = 1'b0;
  assign LCD_BLON       = 1'b0;
  assign LCD_RW         = 1'b0;
  assign LCD_EN         = 1'b0;
  assign LCD_RS         = 1'b0;

  // Turn off SD Card Interface
  assign SD_DAT         = 4'hz;
  assign SD_CMD         = 1'hz;
  assign SD_CLK         = 1'b0;

  // Turn off EEPROM
  assign EEP_I2C_SCLK   = 1'b0;
  assign EEP_I2C_SDAT   = 1'hz;

  // Turn off I2C
  // assign I2C_SDAT       = 1'hz;
  // assign I2C_SCLK       = 1'b0;

  // Turn off VGA
  // assign VGA_BLANK_N    = 1'b0;
  // assign VGA_SYNC_N     = 1'b0;
  // assign VGA_CLK        = 1'b0;
  // assign VGA_HS         = 1'b0;
  // assign VGA_VS         = 1'b0;
  // assign VGA_R          = 8'b00000000;
  // assign VGA_G          = 8'b00000000;
  // assign VGA_B          = 8'b00000000;

  // Turn off Ethernet Interface 0
  assign ENET0_GTX_CLK  = 1'b0;
  assign ENET0_MDC      = 1'b0;
  assign ENET0_MDIO     = 1'hz;
  assign ENET0_RST_N    = 1'b0;
  assign ENET0_TX_DATA  = 4'b0000;
  assign ENET0_TX_EN    = 1'b0;
  assign ENET0_TX_ER    = 1'b0;

  // Turn off Ethernet Interface 1
  assign ENET1_GTX_CLK  = 1'b0;
  assign ENET1_MDC      = 1'b0;
  assign ENET1_MDIO     = 1'hz;
  assign ENET1_RST_N    = 1'b0;
  assign ENET1_TX_DATA  = 4'b0000;
  assign ENET1_TX_EN    = 1'b0;
  assign ENET1_TX_ER    = 1'b0;

  // Turn off HSMC (LVDS)
//  assign HSMC_CLKOUT0   = 1'b0;
//  assign HSMC_D         = 4'hz;
//  assign HSMC_CLKOUT_P1 = 1'b0;
//  assign HSMC_CLKOUT_P2 = 1'b0;
//  assign HSMC_TX_D_P    = 17'b00000000000000000;
//  assign HSMC_CLKOUT_N1 = 1'b0;
//  assign HSMC_CLKOUT_N2 = 1'b0;
//  assign HSMC_TX_D_N    = 17'b00000000000000000;

  // Turn off Audio CODEC
  assign AUD_ADCLRCK    = 1'hz;
  assign AUD_DACLRCK    = 1'hz;
  assign AUD_DACDAT     = 1'b0;
  assign AUD_BCLK       = 1'hz;
  assign AUD_XCK        = 1'b0;

  // Turn off TV Decoder
  // assign TD_RESET_N     = 1'b0;

  // Turn off PS2 Mouse and Keyboard
  assign PS2_CLK        = 1'hz;
  assign PS2_DAT        = 1'hz;
  assign PS2_CLK2       = 1'hz;
  assign PS2_DAT2       = 1'hz;

  // Turn off SMA
  assign SMA_CLKOUT     = 1'b0;

  // Turn off GPIO
  // assign GPIO           = 36'hzzzzzzzzz;

  //--------------------------------------------------------------------
  // Main code starts here
  //--------------------------------------------------------------------

  wire          reset = ~KEY[0];
  wire          change_compute_freq = ~KEY[2];
  wire          game_start = ~KEY[3];

  //--------------------------------------------------------------------
  // Tweaking constants
  //--------------------------------------------------------------------

  localparam pipe_scan_x = 9'd25;
  localparam bird_y = 8'd200;
  localparam bird_threshold = 16'd15;
  localparam pipe_width_offset = 8'd2;
  localparam gap_height = 9'd100;
  localparam pipe_0_threshold = 16'd15;

  wire [8:0]    bird_lower_limit = SW[8:0];
  wire [8:0]    message_on_screen_top = 9'd103;

  //--------------------------------------------------------------------
  //  Audio CODEC and video decoder setting
  //--------------------------------------------------------------------

  I2C_AV_Config config_unit
  (
   //  Host Side
   .iCLK     (CLOCK_50),
   .iRST_N   (~reset),
   //  I2C Side
   .I2C_SCLK (I2C_SCLK),
   .I2C_SDAT (I2C_SDAT)
  );

  //--------------------------------------------------------------------
  // Camera
  //--------------------------------------------------------------------

  wire [9:0]    tv_x;
  wire [9:0]    tv_y;
  wire [15:0]   camera_data;
  wire          camera_data_valid;

  assign TD_RESET_N = ~reset;

  ITU_656_Decoder camera
  (
   .iCLK_27    (TD_CLK27),
   .iRST_N     (~reset),
   .iTD_DATA   (TD_DATA),
   .oTV_X      (tv_x),
   .oTV_Y      (tv_y),
   .oTV_Cont   (),
   .oYCbCr     (camera_data),
   .oDVAL      (camera_data_valid), // High if data is valid
   .iSwap_CbCr (1'b0),
   .iSkip      (1'b0),
   .FVAL       (),
   .Field      ()
  );

  //--------------------------------------------------------------------
  // decoder2m9k Adapter
  //--------------------------------------------------------------------

  wire [16:0]   d2m_adapter_and_m9k_addr;
  wire [7:0]    d2m_adapter_and_m9k_data;
  wire          d2m_adapter_and_m9k_wren;
  wire          writing_useful_location;

  decoder2m9k d2m
  (
   .TV_X                    (tv_x),
   .TV_Y                    (tv_y),
   .iYCbCr                  (camera_data),
   .iDVAL                   (camera_data_valid && SW[17]),
   .m9k_address             (d2m_adapter_and_m9k_addr),
   .m9k_data                (d2m_adapter_and_m9k_data),
   .m9k_wren                (d2m_adapter_and_m9k_wren),
   .offset_y                (8'hcf),
   .offset_x                (8'ha0),
   .writing_useful_location (writing_useful_location)
  );

  //--------------------------------------------------------------------
  // m9k block for Decoder
  //--------------------------------------------------------------------

  wire [16:0]   m9k_and_read_mux_addr;
  wire [7:0]    m9k_and_read_mux_data;

  m9k_block decoder_m9k
  (
   // Camera Side
   .clock_a   (TD_CLK27),
   .address_a (d2m_adapter_and_m9k_addr),
   .data_a    (d2m_adapter_and_m9k_data),
   .wren_a    (d2m_adapter_and_m9k_wren),
   .q_a       (),

   // Computation Side
   .clock_b   (CLOCK_50),
   .address_b (m9k_and_read_mux_addr),
   .data_b    (),
   .wren_b    (1'b0),
   .q_b       (m9k_and_read_mux_data)
  );

  //--------------------------------------------------------------------
  // Compute unit between the two m9k blocks
  //--------------------------------------------------------------------

  wire tapper_done;
  wire compute_go;
  wire reset_compute_and_tapper;

  ThinkControl thinkcontrol
  (
    .clk                      (CLOCK_50),
    .reset                    (reset || change_compute_freq),
    .cycles_between_compute   (24'd5000000),
    .tapper_done              (tapper_done),

    .compute_go               (compute_go),
    .reset_compute_and_tapper (reset_compute_and_tapper)
  );


  wire [9:0] compute_and_bird_seek_m9k_data;
  wire       compute_and_bird_seek_m9k_wren;

  wire [9:0] compute_and_horz_seek_m9k_data;
  wire       compute_and_horz_seek_m9k_wren;

  wire [9:0] compute_and_vert_seek_m9k_data;
  wire       compute_and_vert_seek_m9k_wren;

  wire [8:0] bird_threshold_display;
  wire [8:0] predicted_position;
  wire [8:0] bird_top_x_0;
  wire [8:0] bird_top_x_1;

  wire [2:0] tap_times;
  wire       tap_times_val;

  ComputeUnit #(pipe_width_offset) compute_final
  (
   .clk                            (CLOCK_50),
   .reset                          (reset),

   .compute_go                     (compute_go),

   // Configuration
   .bird_threshold                 (bird_threshold),
   .bird_y                         (bird_y),

   .pipe_0_threshold               (pipe_0_threshold),
   .pipe_scan_x                    (pipe_scan_x),

   .bird_lower_limit               (bird_lower_limit),

   // Camera decoder m9k side
   .decoder_m9k_and_compute_addr   (m9k_and_read_mux_addr),
   .decoder_m9k_and_compute_data   (m9k_and_read_mux_data),

   // VGA m9k side
   .compute_and_decoder_m9k_addr   (compute_and_decoder_m9k_addr),
   .compute_and_decoder_m9k_data   (compute_and_decoder_m9k_data),
   .compute_and_decoder_m9k_wren   (compute_and_decoder_m9k_wren),

   // Output to seek m9k blocks
   .compute_and_bird_seek_m9k_data (compute_and_bird_seek_m9k_data),
   .compute_and_bird_seek_m9k_wren (compute_and_bird_seek_m9k_wren),

   .compute_and_horz_seek_m9k_data (compute_and_horz_seek_m9k_data),
   .compute_and_horz_seek_m9k_wren (compute_and_horz_seek_m9k_wren),

   .compute_and_vert_seek_m9k_data (compute_and_vert_seek_m9k_data),
   .compute_and_vert_seek_m9k_wren (compute_and_vert_seek_m9k_wren),

   .bird_threshold_display         (bird_threshold_display),
   .predicted_position             (predicted_position),
   .bird_top_x_0                   (),
   .bird_top_x_1                   (),

   .tap_times                      (tap_times),
   .tap_times_val                  (tap_times_val),

   .debug                          (),
   .debug2                         (debug2)
  );

  wire [8:0] debug2;
  assign GPIO[24] = debug2[2];
  assign GPIO[26] = debug2[3];
  assign GPIO[28] = debug2[4];
  assign GPIO[30] = debug2[1];
  assign GPIO[32] = debug2[0];

  assign GPIO[34] = tap_times_val;

  Tapper tapper
  (
   .clk                    (CLOCK_50),
   .reset                  (reset || reset_compute_and_tapper),
   .num_tap_cycles         (24'd500000),
   .num_between_tap_cycles (24'd200000),
   .tap_times              (tap_times),
   .tap_times_val          (tap_times_val),
   .tap_out                (GPIO[6]),
   .done                   (tapper_done)
  );

  //--------------------------------------------------------------------
  // m9k block for VGA (essentially a VGA buffer)
  //--------------------------------------------------------------------

  wire [16:0]   compute_and_decoder_m9k_addr;
  wire [7:0]    compute_and_decoder_m9k_data;
  wire          compute_and_decoder_m9k_wren;

  wire [16:0]   m9k_and_m2v_adapter_addr;
  wire [7:0]    m9k_and_m2v_adapter_data;

  m9k_block vga_m9k
  (
   // Computation Side
   .clock_a   (CLOCK_50),
   .address_a (compute_and_decoder_m9k_addr),
   .data_a    (compute_and_decoder_m9k_data),
   .wren_a    (compute_and_decoder_m9k_wren),
   .q_a       (),

   // Vga Side
   .clock_b   (TD_CLK27),
   .address_b (m9k_and_m2v_adapter_addr),
   .data_b    (),
   .wren_b    (1'b0),
   .q_b       (m9k_and_m2v_adapter_data)
  );

  //--------------------------------------------------------------------
  // Seek M9K
  //--------------------------------------------------------------------

  wire          bird_top_valid;
  wire [8:0]    bird_top_x;

  wire          pipe_0_valid;
  wire [7:0]    pipe_0_y;

  wire          pipe_0_gap_top_valid;
  wire [8:0]    pipe_0_gap_top_x;

  wire          dummy;

  seek_m9k bird_seek_m9k
  (
   .clock_a   (CLOCK_50),
   .address_a (1'b0),
   .data_a    (compute_and_bird_seek_m9k_data),
   .wren_a    (compute_and_bird_seek_m9k_wren),
   .q_a       (),

   .clock_b   (TD_CLK27),
   .address_b (1'b0),
   .data_b    (),
   .wren_b    (1'b0),
   .q_b       ({bird_top_valid, bird_top_x})
   );


  seek_m9k horz_seek_m9k
  (
   .clock_a   (CLOCK_50),
   .address_a (1'b0),
   .data_a    (compute_and_horz_seek_m9k_data),
   .wren_a    (compute_and_horz_seek_m9k_wren),
   .q_a       (),

   .clock_b   (TD_CLK27),
   .address_b (1'b0),
   .data_b    (),
   .wren_b    (1'b0),
   .q_b       ({dummy, pipe_0_valid, pipe_0_y})
  );


  seek_m9k vert_seek_m9k
  (
   .clock_a   (CLOCK_50),
   .address_a (1'b0),
   .data_a    (compute_and_vert_seek_m9k_data),
   .wren_a    (compute_and_vert_seek_m9k_wren),
   .q_a       (),

   .clock_b   (TD_CLK27),
   .address_b (1'b0),
   .data_b    (),
   .wren_b    (1'b0),
   .q_b       ({pipe_0_gap_top_valid, pipe_0_gap_top_x})
  );

  wire [8:0]    bird_threshold_display_m9k_out;
  wire          bird_threshold_display_valid;

  seek_m9k bird_threshold_m9k
  (
   .clock_a   (CLOCK_50),
   .address_a (1'b0),
   .data_a    (bird_threshold_display),
   .wren_a    (tap_times_val),
   .q_a       (),

   .clock_b   (TD_CLK27),
   .address_b (1'b0),
   .data_b    (),
   .wren_b    (1'b0),
   .q_b       ({bird_threshold_display_valid, bird_threshold_display_m9k_out})
  );

  wire [8:0]    predicted_position_m9k_out;
  wire          predicted_position_valid;

  seek_m9k predicted_position_m9k
  (
   .clock_a   (CLOCK_50),
   .address_a (1'b0),
   .data_a    (predicted_position),
   .wren_a    (tap_times_val),
   .q_a       (),

   .clock_b   (TD_CLK27),
   .address_b (1'b0),
   .data_b    (),
   .wren_b    (1'b0),
   .q_b       ({predicted_position_valid, predicted_position_m9k_out})
  );

  //--------------------------------------------------------------------
  // m9k2vga Adapter
  //--------------------------------------------------------------------

  wire [18:0]   memory_address_from_vga;
  wire [7:0]    memory_data_to_vga;
  wire          useful_address;
  m9k2vga m2v
  (
   .memory_address_to_m9k   (m9k_and_m2v_adapter_addr),
   .memory_data_from_m9k    (m9k_and_m2v_adapter_data),

   .memory_address_from_vga (memory_address_from_vga),
   .memory_data_to_vga      (memory_data_to_vga),

   // Pipe 0
   .pipe_0_valid            (pipe_0_valid),
   .pipe_0_y                (pipe_0_y),
   .pipe_0_x                (pipe_scan_x),

   // Bird top
   .bird_top_valid          (bird_top_valid),
   .bird_top_y              (bird_y),
   .bird_top_x              (bird_top_x),

   // Pipe 0 gap top
   .pipe_0_gap_top_valid    (pipe_0_gap_top_valid),
   .pipe_0_gap_top_y        (pipe_0_y - pipe_width_offset),
   .pipe_0_gap_top_x        (pipe_0_gap_top_x),

   // Pipe 0 gap bottom
   .pipe_0_gap_btm_valid    (),
   .pipe_0_gap_btm_y        (),
   .pipe_0_gap_btm_x        (),

   .bird_lower_limit        (bird_lower_limit),
   .bird_threshold_display  (bird_threshold_display_m9k_out),
   .history0                (),
   .history1                (),
   .predicted_position      (predicted_position_m9k_out),
   .message_on_screen_top   (message_on_screen_top),

   .useful_address          ()
  );

  //--------------------------------------------------------------------
  // VGA instantiation
  //--------------------------------------------------------------------

  assign VGA_CLK = TD_CLK27;
  VGA_Controller vga_controller
  (
   .clk                (TD_CLK27),
   .reset              (reset),
   .memory_address     (memory_address_from_vga),
   .memory_data        (memory_data_to_vga),
   .vga_r              (VGA_R),
   .vga_g              (VGA_G),
   .vga_b              (VGA_B),
   .vga_h_sync         (VGA_HS),
   .vga_v_sync         (VGA_VS),
   .vga_sync           (VGA_SYNC_N),
   .vga_blank          (VGA_BLANK_N)
  );

  //--------------------------------------------------------------------
  // Visual alive signal/Debug
  //--------------------------------------------------------------------
  assign LEDG[0] = 1'b1;
  assign LEDG[7] = reset;

  assign GPIO[2] = compute_go;
  assign GPIO[4] = tapper_done;

endmodule
